import pymysql
import numpy as np
import pandas as pd
from scipy import optimize as sco


def generate_table(database, table, generate_sql, sql_ip, sql_user, sql_pass, table_comment=''):
    db = pymysql.connect(host=sql_ip, user=sql_user, password=sql_pass, database=database)

    cursor = db.cursor()

    sql = 'create table if not exists `' + table + '` ' + generate_sql + ' comment=\'' + table_comment + '\''
    cursor.execute(sql)
    db.close()


def portfolio_var(weights, cov_matrix):
    return np.dot(weights.T, np.dot(cov_matrix, weights))


def risk_contribute(weights, cov_matrix):
    std = np.sqrt(portfolio_var(weights, cov_matrix))
    mrc = np.matrix(cov_matrix) * np.matrix(weights).T
    return np.multiply(mrc, np.matrix(weights).T) / std


def portfolio_annualised_performance(weights, mean_returns, cov_matrix, freq=252):
    returns = np.sum(mean_returns * weights) * freq
    std = np.sqrt(portfolio_var(weights, cov_matrix)) * np.sqrt(freq)
    return std, returns


def negative_sharpe_ratio(weights, mean_returns, cov_matrix, risk_free_rate, freq=252):
    p_var, p_ret = portfolio_annualised_performance(weights, mean_returns, cov_matrix, freq=freq)
    return -(p_ret - risk_free_rate) / p_var


def max_sharpe_ratio(mean_returns, cov_matrix, risk_free_rate=0.015, freq=252, single_max=0.3):
    num_assets = len(mean_returns)
    args = (mean_returns, cov_matrix, risk_free_rate, freq)
    constraints = ({'type': 'eq', 'fun': lambda x: np.sum(x) - 1})
    # bound = (0.0, 1.0)
    # bounds = tuple(bound for asset in range(num_assets))
    bounds = sco.Bounds([0] * num_assets, [single_max] * num_assets)
    result = sco.minimize(
        negative_sharpe_ratio,
        np.array(num_assets * [1 / num_assets]),
        args=args,
        bounds=bounds,
        constraints=constraints
    )
    return pd.DataFrame(
        result.x,
        index=mean_returns.index,
        columns=['allocation']
    )


def risk_error_sum(weights, cov_matrix):
    # print(weights)
    p_vol = np.sqrt(portfolio_var(weights=weights, cov_matrix=cov_matrix))
    asset_rc = risk_contribute(weights=weights, cov_matrix=cov_matrix)
    return sum(np.abs(asset_rc - p_vol / len(weights)))[0, 0]


def total_weight_constraint(x):
    return np.sum(x) - 1.0


def long_only_constraint(x):
    return x


def risk_parity(cov_matrix, single_max=1, max_i=5000):
    w0 = [0.5] * len(cov_matrix)
    cons = (
        {'type': 'eq', 'fun': total_weight_constraint}
        # {'type': 'ineq', 'fun': long_only_constraint}
    )
    res = sco.minimize(
        risk_error_sum,
        np.array(w0),
        args=cov_matrix,
        constraints=cons,
        bounds=sco.Bounds([0] * len(cov_matrix), [single_max] * len(cov_matrix)),
        options={
            'disp': True,
            'maxiter': max_i
        },
        method='SLSQP',
    )
    return res


if __name__ == '__main__':
    from hbshare.quant.CChen.db_const import sql_write_path_work

    codes = [
        '000001.XSHE',
        '000507.XSHE',
        # '000517.XSHE',
        # '600566.XSHG',
        # '600555.XSHG',
        # '600556.XSHG',
        # '600565.XSHG',
        # '600563.XSHG'
    ]

    data_raw = pd.read_sql_query(
        'select t_date, code, close from a_shares_jq  '
        'where t_date<=20210601 and t_date>=20200101 '
        'and code in ' + str(tuple(codes)),
        sql_write_path_work['daily']
    )
    data = data_raw.pivot(index='t_date', columns='code', values='close')
    returns = data.pct_change()
    mean_returns = returns.mean()
    cov_matrix = returns.cov()

    rp_w = risk_parity(cov_matrix=cov_matrix).x
    rc = risk_contribute(weights=rp_w, cov_matrix=cov_matrix)
    p_vol = np.sqrt(portfolio_var(weights=rp_w, cov_matrix=cov_matrix))
    rb = risk_error_sum(weights=rp_w, cov_matrix=cov_matrix)
    print(mean_returns)
    print(cov_matrix)

    max_sharpe = max_sharpe_ratio(mean_returns=mean_returns, cov_matrix=cov_matrix, single_max=1)
    print(max_sharpe)

    max_sharpe.allocation = [round(i*100, 2)for i in max_sharpe.allocation]
    # max_sharpe = max_sharpe.T

    print(max_sharpe)




"""
SAR Indicator
"""
import datetime

import pandas as pd
import numpy as np
import talib
import os
import hbshare as hbs
from pyecharts import options as opts
from pyecharts.charts import Kline, Scatter
from pyecharts.commons.utils import JsCode
from WindPy import w


w.start()


color_function = """
        function (params) {
            if (params.data.sign > 0) {
                return 'green';
            } else 
            return 'red';
        }
        """

ticker_map = {
    "300450.SZ": "先导智能",
    "688006.SH": "杭可科技",
    "688711.SH": "宏微科技",
    "600201.SH": "生物股份",
    "002371.SZ": "北方华创",
    "300274.SZ": "阳光电源",
    "300750.SZ": "宁德时代"
}


def plot_kline(df, ticker_name):
    kline = (
        Kline(init_opts=opts.InitOpts(width="1600px", height="600px"))
        .add_xaxis(xaxis_data=list(df.index))
        .add_yaxis(
            series_name="klines",
            y_axis=df[['TOPEN', 'TCLOSE', 'LOW', 'HIGH']].values.tolist(),
            itemstyle_opts=opts.InitOpts())
        .set_global_opts(
            legend_opts=opts.LegendOpts(is_show=True, pos_bottom=10, pos_left="center"),
            datazoom_opts=[
                opts.DataZoomOpts(
                    is_show=False,
                    type_="inside",
                    xaxis_index=[0],
                    range_start=80,
                    range_end=100),
                opts.DataZoomOpts(
                    is_show=True,
                    type_="slider",
                    xaxis_index=[0],
                    pos_top="85%",
                    range_start=80,
                    range_end=100)
            ],
            yaxis_opts=opts.AxisOpts(
                is_scale=True,
                splitarea_opts=opts.SplitAreaOpts(
                    is_show=True, areastyle_opts=opts.AreaStyleOpts(opacity=1))
            ),
            tooltip_opts=opts.TooltipOpts(
                trigger="axis",
                axis_pointer_type="cross",
                background_color="rgba(245, 245, 245, 0.8)",
                border_width=1,
                border_color="#ccc",
                textstyle_opts=opts.TextStyleOpts(color="#000")
            ),
            visualmap_opts=opts.VisualMapOpts(
                is_show=False,
                dimension=2,
                series_index=5,
                is_piecewise=True,
                pieces=[
                    {"value": 1, "color": "#00da3c"},
                    {"value": -1, "color": "#ec0000"}
                ]
            ),
            axispointer_opts=opts.AxisPointerOpts(
                is_show=True,
                link=[{"xAxisIndex": "all"}],
                label=opts.LabelOpts(background_color="#777")
            ),
            brush_opts=opts.BrushOpts(
                x_axis_index="all",
                brush_link="all",
                out_of_brush={"colorAlpha": 0.1},
                brush_type="lineX"
            )))

    tmp = df.rename(columns={"sar": "value"})[['value', 'sign']].to_dict(orient="records")

    scatter = (
        Scatter(init_opts=opts.InitOpts(width="1600px", height="600px"))
        .add_xaxis(xaxis_data=list(df.index))
        .add_yaxis(
            series_name="sar",
            # y_axis=df['sar'].values.tolist(),
            y_axis=tmp,
            symbol_size=6,
            label_opts=opts.LabelOpts(is_show=False),
            itemstyle_opts=opts.ItemStyleOpts(color=JsCode(color_function)),
        )
        .set_series_opts()
        .set_global_opts(
            xaxis_opts=opts.AxisOpts(
                type_="category", splitline_opts=opts.SplitLineOpts(is_show=True)
            ),
            yaxis_opts=opts.AxisOpts(
                type_="value",
                axistick_opts=opts.AxisTickOpts(is_show=True),
                splitline_opts=opts.SplitLineOpts(is_show=True),
            ),
            tooltip_opts=opts.TooltipOpts(is_show=False),
            datazoom_opts=[
                opts.DataZoomOpts(
                    is_show=False,
                    type_="inside",
                    xaxis_index=[0],
                    range_start=80,
                    range_end=100),
                opts.DataZoomOpts(
                    is_show=True,
                    type_="slider",
                    xaxis_index=[0],
                    pos_top="85%",
                    range_start=80,
                    range_end=100)
            ]))

    kline.overlap(scatter).render("C:\\Users\\kai.zhang\\Desktop\\SAR跟踪\\标的k线图\\{}.html".format(ticker_name))


def SAR_Monitor(start_date, end_date, ticker, mode="SQL"):
    """
    日线级别的sar预测模块
    """
    if mode == "SQL":
        # 数据库数据
        start_date, end_date = start_date.replace("-", ""), end_date.replace("-", "")
        sql_script = "SELECT TDATE, TOPEN, TCLOSE, HIGH, LOW, VOTURNOVER FROM finchina.CHDQUOTE WHERE " \
                     "SYMBOL = {} and TDATE >= {} and TDATE <= {}".format(ticker.split(".")[0], start_date, end_date)
        res = hbs.db_data_query('readonly', sql_script, page_size=5000)
        data = pd.DataFrame(res['data']).rename(columns={"TDATE": "trade_date"})
        data['trade_date'] = data['trade_date'].astype(str)
        data = data.sort_values(by='trade_date')
    else:
        # Wind数据
        res = w.wsd(ticker, "high,close,low,open", start_date, end_date, "adjDate={};PriceAdj=F".format(
            end_date.replace('-', '')))
        data = pd.DataFrame(res.Data, index=res.Fields, columns=res.Times).T.sort_index()
        data['trade_date'] = data.index
        data['trade_date'] = data['trade_date'].apply(lambda x: datetime.datetime.strftime(x, '%Y%m%d'))
        data = data.rename(columns={"OPEN": "TOPEN", "CLOSE": "TCLOSE"}).sort_values(by='trade_date')
    # SAR
    data = data.set_index('trade_date')
    sar = talib.SAR(data.HIGH, data.LOW)
    data['sar'] = sar
    data = data.dropna()[['TOPEN', 'TCLOSE', 'LOW', 'HIGH', 'sar']].round(2)
    # color
    sign_list = []
    for i in range(data.shape[0]):
        t_date = data.index[i]
        sign = 1 if data.loc[t_date, 'sar'] > data.loc[t_date, 'HIGH'] else -1
        sign_list.append(sign)
    data['sign'] = sign_list
    # SAR图
    plot_kline(data, ticker_map[ticker])
    # 计算收益
    compute_df = data.copy()
    compute_df['ret'] = compute_df['TOPEN'].pct_change().shift(-1)
    compute_df.loc[end_date.replace('-', ''), 'ret'] = \
        compute_df.loc[end_date.replace('-', ''), 'TCLOSE'] / compute_df.loc[end_date.replace('-', ''), 'TOPEN'] - 1
    compute_df['ret_port'] = compute_df['ret']
    compute_df['pre_sign'] = compute_df['sign'].shift(1)
    compute_df.loc[compute_df['pre_sign'] != -1, 'ret_port'] = 0
    compute_df['股价走势'] = (1 + compute_df['ret']).cumprod()
    compute_df['SAR择时组合走势'] = (1 + compute_df['ret_port']).cumprod()
    # (1 + compute_df[['ret', 'ret_port']]).cumprod().plot.line()

    include_cols = ['TOPEN', 'HIGH', 'LOW', 'TCLOSE', 'sar', 'sign', 'pre_sign', '股价走势', 'SAR择时组合走势']

    return compute_df[include_cols]


def sar_summary():
    path = "C:\\Users\\kai.zhang\\Desktop\\SAR跟踪"
    compute_dict = pd.read_excel(os.path.join(path, "SAR跟踪.xlsx"), sheet_name=None)
    ticker_name_list = list(compute_dict.keys())
    summary_df = pd.DataFrame(index=ticker_name_list, columns=['SAR', '最新交易日收盘价', '最新信号', '前一日信号',
                                                               '做多天数', '做空天数', "近一年波动率",
                                                               '2022年收益', '2022年原始收益',
                                                               '2023年至今收益', '2023年原始收益'])
    sign_list = []
    for ticker_name in ticker_name_list:
        compute_df = compute_dict[ticker_name].copy()
        # compute_df['trade_date'] = compute_df['trade_date'].apply(lambda x: datetime.datetime.strftime(x, '%Y%m%d'))
        compute_df['trade_date'] = compute_df['trade_date'].astype(str)
        sign_list.append(
            compute_df.set_index('trade_date').rename(columns={"sign": ticker_name})[[ticker_name]])
        summary_df.loc[ticker_name, "SAR"] = compute_df['sar'].tolist()[-1]
        summary_df.loc[ticker_name, "最新交易日收盘价"] = compute_df['TCLOSE'].tolist()[-1]
        summary_df.loc[ticker_name, "最新信号"] = compute_df['sign'].tolist()[-1]
        summary_df.loc[ticker_name, "前一日信号"] = compute_df['pre_sign'].tolist()[-1]
        summary_df.loc[ticker_name, "做多天数"] = compute_df['pre_sign'].value_counts().loc[-1]
        summary_df.loc[ticker_name, "做空天数"] = compute_df['pre_sign'].value_counts().loc[1]
        summary_df.loc[ticker_name, "近一年波动率"] = format(
            compute_df['TCLOSE'].pct_change()[-252:].std() * np.sqrt(252), '.2%')

        compute_df = compute_df.set_index('trade_date')
        tmp = compute_df.loc["20221230", ['股价走势', 'SAR择时组合走势']] / \
            compute_df.loc["20220104", ['股价走势', 'SAR择时组合走势']] - 1
        summary_df.loc[ticker_name, '2022年收益'] = format(tmp.loc['SAR择时组合走势'], '.2%')
        summary_df.loc[ticker_name, '2022年原始收益'] = format(tmp.loc['股价走势'], '.2%')
        tmp = compute_df.iloc[-1][['股价走势', 'SAR择时组合走势']] / \
            compute_df.loc["20221230", ['股价走势', 'SAR择时组合走势']] - 1
        summary_df.loc[ticker_name, '2023年至今收益'] = format(tmp.loc['SAR择时组合走势'], '.2%')
        summary_df.loc[ticker_name, '2023年原始收益'] = format(tmp.loc['股价走势'], '.2%')

    sign_df = pd.concat(sign_list, axis=1)

    return summary_df, sign_df


if __name__ == '__main__':
    # 分钟线数据
    # res = w.wsi("688006.SH", "open,close,high,low", "2022-01-01 09:00:00", "2022-06-19 21:30:32", "BarSize=60")
    # data = pd.DataFrame(res.Data, index=res.Fields, columns=res.Times).T.sort_index()
    # data = data.rename(columns={"open": "TOPEN", "close": "TCLOSE", "high": "HIGH", "low": "LOW"})
    # sar = talib.SAR(data.HIGH, data.LOW)
    # data['sar'] = sar
    # data = data.dropna()[['TOPEN', 'TCLOSE', 'LOW', 'HIGH', 'sar']].round(2)

    # 日k
    s_date = "2021-12-31"
    e_date = "2023-04-12"
    excel_writer = pd.ExcelWriter("C:\\Users\\kai.zhang\\Desktop\\SAR跟踪\\SAR跟踪.xlsx", engine='openpyxl')
    for test_ticker in ticker_map.keys():
        t_df = SAR_Monitor(s_date, e_date, test_ticker, mode="Wind")
        t_df.to_excel(excel_writer, sheet_name=ticker_map[test_ticker])
    excel_writer.close()

    results = sar_summary()


import datetime
import pandas as pd
import numpy as np
from hbshare.fe.XZ import db_engine
from hbshare.fe.XZ import functionality
from hbshare.fe.customer_trading_analysis import functions

util=functionality.Untils()
hbdb=db_engine.HBDB()
localdb=db_engine.PrvFunDB().engine
plot=functionality.Plot(1200,600)


if __name__ == '__main__':


    # stock_centralization=pd.read_excel(r"E:\GitFolder\hbshare\fe\customer_trading_analysis\公募基金个股权重时序变化.xlsx")
    # market_centralization=pd.DataFrame(index=stock_centralization['日期'].unique().tolist(),columns=['抱团力度'])
    # for date in stock_centralization['日期'].unique().tolist():
    #     tempdf=stock_centralization[stock_centralization['日期']==date]
    #     market_centralization.loc[date,'抱团力度']=tempdf.nlargest(5,'占净值比')['占净值比'].sum()+\
    #                                            tempdf.nlargest(10,'占净值比')['占净值比'].sum()\
    #                                            +tempdf.nlargest(20,'占净值比')['占净值比'].sum()
    # market_centralization.index=market_centralization.index.astype(str)
    # plot.plotly_line_style(market_centralization.sort_index(),'公募基金抱团情况')

    # from hbshare.fe.mutual_analysis import  nav_based as nb
    #
    #
    # index_month_ret=nb.get_monthly_index_ret('930950',
    #                                          start_date='20151231',
    #                                          end_date='20230131')[['tjyf','hb1y','rqzh']].rename(columns={'hb1y':'930950'})
    #
    # month_list=index_month_ret[['tjyf','rqzh']]
    # index_month_ret.set_index('tjyf',inplace=True)
    # count=6
    # jjdm_list=[]
    # for i in range(1,len(month_list)):
    #     if(count%6==0):
    #         jjdm_list=util.get_mutual30_bmk(month_list['rqzh'].iloc[i-1])
    #     sql="select avg(hb1y) as mutual_30_bmk from st_fund.t_st_gm_yhb where jjdm in ({0}) and tjyf='{1}' and hb1y!=99999 "\
    #         .format(util.list_sql_condition(jjdm_list),month_list['tjyf'].iloc[i])
    #     jj_month_ret=hbdb.db2df(sql,db='funduser')
    #     index_month_ret.loc[month_list['tjyf'].iloc[i],'公募30bmk']=jj_month_ret.values[0]
    #     count+=1
    #
    # for index_name in ['885001','881001','399370','399371','399314','399315','399316']:
    #     index_month_ret = pd.merge(index_month_ret, nb.get_monthly_index_ret(index_name,
    #                                                                          start_date='20151231',
    #                                                                          end_date='20230131')[
    #         ['tjyf', 'hb1y']].rename(columns={'hb1y': index_name}),
    #                                how='inner', on='tjyf')
    #
    # index_month_ret.drop('rqzh',axis=1,inplace=True)
    # index_ret_rank=\
    #     index_month_ret[['930950','885001','公募30bmk']].rank(axis=1,ascending=False).iloc[1:]
    # index_ret_rank=\
    #     pd.concat([(index_ret_rank == 1).sum() / len(index_ret_rank),index_ret_rank.mean(axis=0)],axis=1).rename(columns={0:'月度胜率',1:'月度平均排名'})
    # index_ret_rank['月度胜率']=index_ret_rank['月度胜率'].map("{:.2%}".format)
    # index_ret_rank['月度平均排名']=index_ret_rank['月度平均排名'].map("{:.2}".format)
    # plot.plotly_table(index_ret_rank.reset_index().rename(columns={'index':'对应指数'}),
    #                   300,'')
    #
    # name_map=dict(zip(['881001','399370','399371','399314','399315','399316']
    #                   ,['万得全A','巨潮成长','巨潮价值','巨潮大盘','巨潮中盘','巨潮小盘']))
    # index_month_ret.rename(columns=name_map,inplace=True)
    # index_month_ret.set_index('tjyf',inplace=True)
    #
    # index_corr=index_month_ret.corr()
    # index_str=(index_month_ret/100).std()*np.sqrt(12)
    #
    # index_month_ret=index_month_ret/100+1
    #
    #
    # index_year_ret=index_month_ret.copy()
    # index_year_ret['year'] = index_year_ret.index.astype(str).str[0:4]
    # index_year_ret=index_year_ret.groupby('year').cumprod()
    # index_year_ret['year'] = index_year_ret.index.astype(str).str[0:4]
    # index_year_ret=\
    #     (index_year_ret.drop_duplicates('year',keep='last').drop('year',axis=1)-1).iloc[1:]
    # for col in index_year_ret.columns:
    #     index_year_ret[col]=index_year_ret[col].map("{:.2%}".format)
    #
    #
    # for col in index_corr.columns:
    #     index_corr[col]=index_corr[col].map("{:.2%}".format)
    # plot.plotly_table(index_corr.reset_index().rename(columns={'index':'指数收益率相关性'}),800,'')
    #
    # index_year_ret.index = index_year_ret.index.astype(str).str[0:4]
    # plot.plotly_table(index_year_ret.reset_index().rename(columns={'tjyf':'年份'}),800,'')
    #
    # index_month_ret.iloc[0] = 1
    # index_month_ret.index=index_month_ret.index.astype(str)
    #
    #
    # index_month_ret=index_month_ret.cumprod()
    # index_annual_ret=np.power((index_month_ret.iloc[-1]),12/(len(index_month_ret)-1))-1
    # index_annual_ret=(index_annual_ret ).map("{:.2%}".format)
    # plot.plotly_table(index_annual_ret.to_frame('年化收益率').T.reset_index().rename(columns={'index':''}),800,'')
    # plot.plotly_table(index_str.map("{:.2%}".format).to_frame('年化波动率').T.reset_index().rename(columns={'index':''}),800,'')
    # plot.plotly_line_style(index_month_ret[['930950','公募30bmk','885001','万得全A']],'指数净值时序图')
    #
    #
    # output_df=pd.DataFrame(index=['价值','均衡','成长'])
    # output_df2=[]
    # output_df3=[]
    # output_df33=[]
    # output_df4=[]
    # output_df44=[]
    # output_df111=[]
    # output_df333=[]
    # output_df444=[]
    #
    # date_list=['20150930','20151231','20160331','20160630','20160930','20161230',
    #            '20170331','20170630','20170929','20171229','20180330','20180629','20180928','20181228',
    #            '20190329','20190628','20190930','20191231','20200331','20200630','20200930','20201231',
    #            '20210331','20210630','20210930','20211231','20220331','20220630','20221231']
    #
    # jj_num_summary=[]
    # jj_mv_summary=[]
    #
    # for date in date_list:
    #     sql="select jjdm,jjmc,clrq,jjjc,ejfl from st_fund.t_st_gm_jjxx where ejfl in ('13','15','16','37','35') and clrq<='{}'  "\
    #         .format(date)
    #     jjdm_list=hbdb.db2df(sql,db='funduser').sort_values(['clrq','jjdm'])
    #     jjdm_list = jjdm_list[~jjdm_list['jjjc'].str.contains('C')]
    #     jjdm_list.drop_duplicates(['jjmc'],keep='first',inplace=True)
    #
    #     jjdm_list['ejfl']= \
    #         ((((jjdm_list['ejfl'].replace('13','普通股票型')).replace('15','指数增强型')).replace('16','被动指数型')).replace('37','偏股混合型')).replace('35','灵活配置型')
    #     jj_num_summary.append(jjdm_list.groupby('ejfl').count()['jjdm'].to_frame(date).T)
    #
    #     sql="select jjdm,jjjzc from st_fund.t_st_gm_zcpz where jjdm in ({0}) and jsrq>'{1}' and jsrq<='{2}'"\
    #         .format(util.list_sql_condition(jjdm_list['jjdm'].tolist()),date[0:6]+'01',date[0:6]+'31')
    #     jj_size=hbdb.db2df(sql,db='funduser')
    #     jjdm_list=pd.merge(jjdm_list,jj_size,how='left',on='jjdm').fillna(0)
    #     jj_mv_summary.append(jjdm_list.groupby('ejfl').sum()['jjjzc'].to_frame(date).T)
    #
    # jj_mv_summary=pd.concat(jj_mv_summary, axis=0)/100000000
    # jj_num_summary = pd.concat(jj_num_summary, axis=0)
    #
    # plot.plotly_table(jj_mv_summary.round(2).reset_index().rename(columns={'index':'日期'}),400,'')
    # plot.plotly_table(jj_num_summary.reset_index().rename(columns={'index':'日期'}), 400, '')
    #
    # jj_mv_summary['year'] = jj_mv_summary.index.astype(str).str[0:4]
    # index_year_ret=pd.merge(index_year_ret,
    #          jj_mv_summary.drop_duplicates('year'
    #                                        , keep='last').set_index('year').sum(axis=1).to_frame('公募基金整体规模（亿元)')
    #          ,how='left',left_index=True,right_index=True)
    # plot.plotly_table(index_year_ret.reset_index().rename(columns={'tjyf':'年份'}),1400,'')


    # data,layout=plot.plotly_area(jj_num_summary.T,'各类基金数量时序统计'
    #                              ,range_upper=jj_num_summary.sum(axis=1).max(),if_percentage=False)
    # plot.plot_render(data,layout)
    # data,layout=plot.plotly_area(jj_mv_summary.T,'各类基金市值时序统计(亿元）'
    #                              ,range_upper=jj_mv_summary.sum(axis=1).max(),if_percentage=False)
    # plot.plot_render(data, layout)
    #
    # for i in range(1,len(date_list)):
    #
    #     end_date=date_list[i]
    #
    #     sql="select jjdm,jjjc from st_fund.t_st_gm_jjxx where ejfl in ('13','37') and clrq<='{}'  "\
    #         .format(date_list[i-1])
    #     jj_pool=hbdb.db2df(sql,db='funduser')
    #
    #     sql="select jjdm,jjjc,clrq from st_fund.t_st_gm_jjxx where cpfl='2' and ejfl='37' and clrq<='{}'  "\
    #         .format(date_list[i-1])
    #     jj_pool3=hbdb.db2df(sql,db='funduser').sort_values(['clrq','jjdm'])
    #     jj_pool3=jj_pool3[~jj_pool3['jjjc'].str.contains('C')]
    #     jj_pool3.drop_duplicates(['jjjc'],keep='first',inplace=True)
    #
    #     if(i==1):
    #         sql = "select jjdm,jjjc,clrq  from st_fund.t_st_gm_jjxx where cpfl='2' and (ejfl='13' or  ejfl='37' or ejfl='15' or ejfl='16') and jjzt='0' and clrq<='{}'  " \
    #             .format(date_list[i - 1])
    #         jj_pool4 = hbdb.db2df(sql, db='funduser')
    #     else:
    #         sql = "select jjdm,jjjc,clrq  from st_fund.t_st_gm_jjxx where cpfl='2' and (ejfl='13' or  ejfl='37' or ejfl='15' or ejfl='16') and jjzt='0' and clrq<='{}'  " \
    #             .format(date_list[i - 2])
    #         jj_pool4 = hbdb.db2df(sql, db='funduser')
    #     jj_pool4=hbdb.db2df(sql,db='funduser').sort_values(['clrq','jjdm'])
    #     jj_pool4=jj_pool4[~jj_pool4['jjjc'].str.contains('C')]
    #     jj_pool4.drop_duplicates(['jjjc'],keep='first',inplace=True)
    #
    #     sql="select jjdm,jjjzc from st_fund.t_st_gm_zcpz where jjdm in ({0}) and jsrq>'{1}' and jsrq<='{2}'"\
    #         .format(util.list_sql_condition(jj_pool['jjdm'].tolist()),date_list[i-1],end_date[0:6]+'31')
    #     jj_size=hbdb.db2df(sql,db='funduser')
    #     jj_pool=pd.merge(jj_pool,jj_size,how='left',on='jjdm')
    #     jj_pool=jj_pool[jj_pool['jjjzc'].notnull()]
    #
    #     sql="select jsrq,jjdm,yjxymc,zjbl from hbs_industry_class1_exp where  jsrq='{0}'"\
    #         .format(end_date)
    #     industry_data=pd.read_sql(sql,con=localdb).pivot_table('zjbl','jjdm','yjxymc').fillna(0)
    #
    #     sql="SELECT * from hbs_style_exp where jsrq='{}' "\
    #         .format(end_date)
    #     style_data=pd.read_sql(sql,con=localdb).pivot_table('zjbl','jjdm','style_type')
    #     sql="SELECT * from hbs_size_exp where jsrq='{}' "\
    #         .format(end_date)
    #     size_data=pd.read_sql(sql,con=localdb).pivot_table('zjbl','jjdm','size_type')
    #
    #     jj_pool33=pd.merge(jj_pool3, style_data, how='left', on='jjdm')
    #     jj_pool33 = jj_pool33[jj_pool33[['价值','均衡','成长']].sum(axis=1) != 0]
    #     jj_pool33=jj_pool33[['价值','均衡','成长']].sum(axis=0)/len(jj_pool33)
    #     output_df33.append(jj_pool33.to_frame(end_date))
    #
    #     jj_pool333=pd.merge(jj_pool3, size_data, how='left', on='jjdm')
    #     jj_pool333 = jj_pool333[jj_pool333[['中盘','大盘','小盘']].sum(axis=1) != 0]
    #     jj_pool333=jj_pool333[['中盘','大盘','小盘']].sum(axis=0)/len(jj_pool333)
    #     output_df333.append(jj_pool333.to_frame(end_date))
    #
    #     jj_pool3=pd.merge(jj_pool3,industry_data, how='left', on='jjdm')
    #     jj_pool3=jj_pool3[jj_pool3[industry_data.columns.tolist()].sum(axis=1)!=0]
    #     jj_pool3=jj_pool3[industry_data.columns.tolist()].sum(axis=0) / len(jj_pool3)
    #     output_df3.append(jj_pool3.to_frame(end_date))
    #
    #     jj_pool44=pd.merge(jj_pool4, style_data, how='left', on='jjdm')
    #     jj_pool44 = jj_pool44[jj_pool44[['价值','均衡','成长']].sum(axis=1) != 0]
    #     jj_pool44=jj_pool44[['价值','均衡','成长']].sum(axis=0)/len(jj_pool44)
    #     output_df44.append(jj_pool44.to_frame(end_date))
    #
    #     jj_pool444=pd.merge(jj_pool4, size_data, how='left', on='jjdm')
    #     jj_pool444 = jj_pool444[jj_pool444[['中盘','大盘','小盘']].sum(axis=1) != 0]
    #     jj_pool444=jj_pool444[['中盘','大盘','小盘']].sum(axis=0)/len(jj_pool444)
    #     output_df444.append(jj_pool444.to_frame(end_date))
    #
    #     jj_pool4 = pd.merge(jj_pool4, industry_data, how='left', on='jjdm')
    #     jj_pool4=jj_pool4[jj_pool4[industry_data.columns.tolist()].sum(axis=1)!=0]
    #     jj_pool4=jj_pool4[industry_data.columns.tolist()].sum(axis=0) / len(jj_pool4)
    #     output_df4.append(jj_pool4.to_frame(end_date))
    #
    #
    #     jj_pool2 = pd.merge(jj_pool, industry_data, how='left', on='jjdm')
    #     jj_pool2=jj_pool2[jj_pool2[industry_data.columns.tolist()].sum(axis=1)!=0]
    #     market_total_jjjzc=jj_pool2['jjjzc'].sum(axis=0)
    #     for col in industry_data.columns:
    #         jj_pool2[col]=jj_pool2[col]*jj_pool2['jjjzc']/market_total_jjjzc
    #     jj_pool2.drop(['jjdm','jjjc','jjjzc'],axis=1,inplace=True)
    #     output_df2.append(jj_pool2.sum(axis=0).to_frame(end_date))
    #
    #     jj_pool111=pd.merge(jj_pool, size_data, how='left', on='jjdm')
    #     jj_pool111 = jj_pool111[jj_pool111[['中盘','大盘','小盘']].sum(axis=1) != 0]
    #     market_total_jjjzc=jj_pool111['jjjzc'].sum(axis=0)
    #     for col in ['中盘','大盘','小盘']:
    #         jj_pool111[col]=jj_pool111[col]*jj_pool111['jjjzc']/market_total_jjjzc
    #     output_df111.append(jj_pool111[['中盘','大盘','小盘']].sum(axis=0).to_frame(end_date))
    #
    #     jj_pool = pd.merge(jj_pool, style_data, how='left', on='jjdm')
    #     jj_pool=jj_pool[jj_pool[['价值','均衡','成长']].sum(axis=1)!=0]
    #     market_total_jjjzc=jj_pool['jjjzc'].sum()
    #     for col in ['价值','均衡','成长']:
    #         jj_pool[col]=jj_pool[col]*jj_pool['jjjzc']/market_total_jjjzc
    #     output_df[end_date]=jj_pool[['价值','均衡','成长']].sum(axis=0)
    #
    # industry_summary=[]
    # industry_latest=[]
    # style_summary=[]
    # style_latest=[]
    # size_summary=[]
    # size_latest=[]
    # style_vol=pd.DataFrame()
    # size_vol = pd.DataFrame()
    #
    # output_df2=pd.concat(output_df2, axis=1)
    # industry_latest.append(output_df2['20220630'].map("{:.3}".format).to_frame('930950'))
    # industry_summary.append(output_df2.mean(axis=1).map("{:.3}".format).to_frame('930950'))
    # data,layout=plot.plotly_area(output_df2,'930950_行业分布')
    # plot.plot_render(data,layout)
    # output_df2.to_excel('930950_行业分布.xlsx')
    #
    # output_df3=pd.concat(output_df3, axis=1)
    # industry_latest.append(output_df3['20220630'].map("{:.3}".format).to_frame('885001'))
    # industry_summary.append(output_df3.mean(axis=1).map("{:.3}".format).to_frame('885001'))
    # data,layout=plot.plotly_area(output_df3,'885001_行业分布')
    # plot.plot_render(data,layout)
    # output_df3.to_excel('885001_行业分布.xlsx')
    #
    # output_df4=pd.concat(output_df4, axis=1)
    # industry_latest.append(output_df4['20220630'].map("{:.3}".format).to_frame('公募30Bmk'))
    # industry_summary.append(output_df4.mean(axis=1).map("{:.3}".format).to_frame('公募30Bmk'))
    # data,layout=plot.plotly_area(output_df4,'公募30Bmk_行业分布')
    # plot.plot_render(data,layout)
    # output_df4.to_excel('公募30Bmk_行业分布.xlsx')
    #
    # output_df33=pd.concat(output_df33, axis=1)
    # style_vol['885001']=\
    #     ["{:.2%}".format(((output_df33.T/100).cov()*np.sqrt(12))['价值'].sum()+\
    #     ((output_df33.T/100).cov()*np.sqrt(12))['均衡'].iloc[1:].sum()+\
    #     ((output_df33.T/100).cov()*np.sqrt(12))['成长'].iloc[2:].sum())]
    # style_summary.append(output_df33.mean(axis=1).map("{:.3}".format).to_frame('885001'))
    # style_latest.append(output_df33['20220630'].map("{:.3}".format).to_frame('885001'))
    # data,layout=plot.plotly_area(output_df33,'885001_风格分布')
    # plot.plot_render(data,layout)
    # output_df33.to_excel('885001_风格分布.xlsx')
    #
    # output_df44=pd.concat(output_df44, axis=1)
    # style_vol['公募30Bmk']=\
    #     ["{:.2%}".format(((output_df44.T/100).cov()*np.sqrt(12))['价值'].sum()+\
    #     ((output_df44.T/100).cov()*np.sqrt(12))['均衡'].iloc[1:].sum()+\
    #     ((output_df44.T/100).cov()*np.sqrt(12))['成长'].iloc[2:].sum())]
    # style_summary.append(output_df44.mean(axis=1).map("{:.3}".format).to_frame('公募30Bmk'))
    # style_latest.append(output_df44['20220630'].map("{:.3}".format).to_frame('公募30Bmk'))
    # data,layout=plot.plotly_area(output_df44,'公募30Bmk_风格分布')
    # plot.plot_render(data,layout)
    # output_df44.to_excel('公募30Bmk_风格分布.xlsx')
    #
    # style_vol['930950']=\
    #     ["{:.2%}".format(((output_df.T/100).cov()*np.sqrt(12))['价值'].sum()+\
    #     ((output_df.T/100).cov()*np.sqrt(12))['均衡'].iloc[1:].sum()+\
    #     ((output_df.T/100).cov()*np.sqrt(12))['成长'].iloc[2:].sum())]
    # style_summary.append(output_df.mean(axis=1).map("{:.3}".format).to_frame('930950'))
    # style_latest.append(output_df['20220630'].map("{:.3}".format).to_frame('930950'))
    # data,layout=plot.plotly_area(output_df,'930950_风格分布')
    # plot.plot_render(data,layout)
    # output_df.to_excel('930950_风格分布.xlsx')
    #
    #
    # output_df333=pd.concat(output_df333, axis=1)
    # size_vol['885001']=\
    #     ["{:.2%}".format(((output_df333.T/100).cov()*np.sqrt(12))['中盘'].sum()+\
    #     ((output_df333.T/100).cov()*np.sqrt(12))['大盘'].iloc[1:].sum()+\
    #     ((output_df333.T/100).cov()*np.sqrt(12))['小盘'].iloc[2:].sum())]
    # size_summary.append(output_df333.mean(axis=1).map("{:.3}".format).to_frame('885001'))
    # size_latest.append(output_df333['20220630'].map("{:.3}".format).to_frame('885001'))
    # data,layout=plot.plotly_area(output_df333,'885001_市值风格分布')
    # plot.plot_render(data,layout)
    # output_df333.to_excel('885001_市值分布.xlsx')
    #
    # output_df444=pd.concat(output_df444, axis=1)
    # size_vol['公募30Bmk']=\
    #     ["{:.2%}".format(((output_df444.T/100).cov()*np.sqrt(12))['中盘'].sum()+\
    #     ((output_df444.T/100).cov()*np.sqrt(12))['大盘'].iloc[1:].sum()+\
    #     ((output_df444.T/100).cov()*np.sqrt(12))['小盘'].iloc[2:].sum())]
    # size_summary.append(output_df444.mean(axis=1).map("{:.3}".format).to_frame('公募30Bmk'))
    # size_latest.append(output_df444['20220630'].map("{:.3}".format).to_frame('公募30Bmk'))
    # data,layout=plot.plotly_area(output_df444,'公募30Bmk_市值风格分布')
    # plot.plot_render(data,layout)
    # output_df444.to_excel('公募30Bmk_市值分布.xlsx')
    #
    # output_df111 = pd.concat(output_df111, axis=1)
    # size_vol['930950']=\
    #     ["{:.2%}".format(((output_df111.T/100).cov()*np.sqrt(12))['中盘'].sum()+\
    #     ((output_df111.T/100).cov()*np.sqrt(12))['大盘'].iloc[1:].sum()+\
    #     ((output_df111.T/100).cov()*np.sqrt(12))['小盘'].iloc[2:].sum())]
    # size_summary.append(output_df111.mean(axis=1).map("{:.3}".format).to_frame('930950'))
    # size_latest.append(output_df111['20220630'].map("{:.3}".format).to_frame('930950'))
    # data,layout=plot.plotly_area(output_df111,'930950_市值风格分布')
    # plot.plot_render(data,layout)
    # output_df111.to_excel('930950_市值分布.xlsx')
    #
    #
    #
    # plot.plotly_table(pd.concat(style_summary,axis=1).reset_index().rename(columns={'index':'风格类型'}),400,'')
    # plot.plotly_table(pd.concat(industry_summary,axis=1).reset_index().rename(columns={'index':'行业'}),400,'')
    # plot.plotly_table(pd.concat(size_summary, axis=1).reset_index().rename(columns={'index': '市值风格类型'}), 400, '')
    # plot.plotly_table(pd.concat(style_latest,axis=1).reset_index().rename(columns={'index':'风格类型'}),400,'')
    # plot.plotly_table(pd.concat(industry_latest,axis=1).reset_index().rename(columns={'index':'行业'}),400,'')
    # plot.plotly_table(pd.concat(size_latest, axis=1).reset_index().rename(columns={'index': '市值风格类型'}), 400, '')
    # style_vol.index = ['波动率']
    # plot.plotly_table(style_vol.reset_index().rename(columns={'index':''}),400,'')
    # size_vol.index = ['波动率']
    # plot.plotly_table(size_vol.reset_index().rename(columns={'index':''}),400,'')



    #set parameters:
    name='汉和'
    file_name='汉和交易数据'


    #'SX9799'
    #get fund basic information
    jjdm_list=functions.get_jjdm_list_by_name(name)
    fund_baisc_info=\
        functions.get_fund_basic_info(jjdm_list=jjdm_list)
    fund_baisc_info['销售标志']=fund_baisc_info['销售标志'].astype(int)
    jjdm_list=fund_baisc_info[fund_baisc_info['上线日期'] >= '19000101']['jjdm'].tolist()


    #get fund nav data
    fund_nav=\
        functions.get_fund_nav(jjdm_list=jjdm_list)\
            .pivot_table('ljjz','jzrq','jjdm')

    #get fund trading data
    trade_data=functions.get_customer_trade_date(file_name=file_name)
    trade_data=functions.transfor_tarde_data(trade_data)
    fund_baisc_info.loc[fund_baisc_info['上线日期'] == '19000101','上线日期']=str(trade_data['交易日期'].min())
    online_date=fund_baisc_info[fund_baisc_info['上线日期'] >= '19000101'].pivot_table('销售标志', '上线日期', '基金简称')

    buy_time,sell_time,buy_return,sell_return,\
    buy_winning_duration_summary,sell_winning_duration_summary,\
    fund_nav,online_time_summary,max_buy_day,max_sell_day,nav_jjdm=\
        functions.customer_behavior_analysis(fund_nav,trade_data,online_date,fund_baisc_info,name)

    functions.plot_customer_behavior_analysis(online_time_summary, buy_time, sell_time,
                                    buy_return, sell_return, buy_winning_duration_summary,
                                    sell_winning_duration_summary)

    customer_financial_data,latest_customer_return_dist,\
    latest_customer_mv_dist,customer_holding_time,left_customers_ret_stats,outstandingbuy_customer_return_dist\
        ,outstandingbuy_customer_mv_dist,outstandingsell_customer_return_dist,outstandingsell_customer_mv_dist,\
    customer_state_summary=\
        functions.customer_financial_analysis(fund_nav,trade_data,max_buy_day,max_sell_day)

    functions.plot_customer_financial_analysis(customer_holding_time,left_customers_ret_stats,customer_financial_data,
                                     latest_customer_return_dist,latest_customer_mv_dist,fund_nav,trade_data
                                    ,outstandingbuy_customer_return_dist,outstandingbuy_customer_mv_dist,
                                    outstandingsell_customer_return_dist,
                                               outstandingsell_customer_mv_dist,
                                               pd.concat(customer_state_summary,axis=0).groupby('交易日期').sum(),fund_baisc_info,nav_jjdm,name)

